/*
 * platform_mrfl_regulator.c - Merrifield regulator machine drvier
 * Copyright (c) 2012, Intel Corporation.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <linux/platform_device.h>
#include <linux/regulator/intel_basin_cove_pmic.h>
#include <linux/regulator/machine.h>
#include <linux/gpio.h>
#include <linux/regulator/fixed.h>
#include <linux/board_asustek.h>

#include <asm/intel-mid.h>

/***********VPROG3 REGUATOR platform data*************/
static struct regulator_consumer_supply vprog3_consumer[] = {
	REGULATOR_SUPPLY("vqmmc", "0000:00:01.2"),
};
static struct regulator_init_data vprog3_data = {
	.constraints = {
		/*
		 * FIXME: AON is due to cd pin cannot be stablized.
		 *	  VCCA and VCCB need to AON so that cd pin
		 *	  can be stablized.
		 *
		 *	  .always_on		= 1,
		 */
		.min_uV			= 1050000,
		.max_uV			= 2900000,
		.valid_ops_mask		= REGULATOR_CHANGE_STATUS
						| REGULATOR_CHANGE_VOLTAGE,
		.valid_modes_mask	= REGULATOR_MODE_NORMAL,
		.initial_mode		= REGULATOR_MODE_STANDBY,
	},
	.num_consumer_supplies	= ARRAY_SIZE(vprog3_consumer),
	.consumer_supplies	= vprog3_consumer,
};
static struct intel_pmic_info vprog3_info = {
	.pmic_reg   = VPROG3CNT_ADDR,
	.init_data  = &vprog3_data,
	.table_len  = ARRAY_SIZE(VPROG3_VSEL_table),
	.table      = VPROG3_VSEL_table,
};
static struct platform_device vprog3_device = {
	.name = "intel_regulator",
	.id = VPROG3,
	.dev = {
		.platform_data = &vprog3_info,
	},
};

/* VDD3_SD_SW regulator platform data */
static struct regulator_consumer_supply gpio_en_3v_consumers[] = {
	REGULATOR_SUPPLY("vmmc", "0000:00:01.2"),
};

struct regulator_init_data gpio_en_3v_regulator = {
	.constraints = {
		.name = "VDD3-SD-SW",
		.min_uV = 3000000,
		.max_uV = 3000000,
		.valid_ops_mask = REGULATOR_CHANGE_STATUS,
	},
	.num_consumer_supplies = ARRAY_SIZE(gpio_en_3v_consumers),
	.consumer_supplies = gpio_en_3v_consumers,
};

#define MOOR_SD_PWR_EN 118
static struct fixed_voltage_config anniedale_gpio_sd_regulator_data = {
	.supply_name            = "VDD3-SD-SW",
	.gpio                   = MOOR_SD_PWR_EN,
	.microvolts             = 3000000,
	.enable_high            = 1,
	.init_data              = &gpio_en_3v_regulator,
	.startup_delay          = 5000, /* 1200us */
};

static struct platform_device anniedale_gpio_sd_regulator_dev = {
	.name   = "reg-fixed-voltage",
	.id     = 1,
	.dev    = {
		.platform_data  = &anniedale_gpio_sd_regulator_data,
	},
};

static struct platform_device *regulator_devices_NXP[] __initdata = {
	&anniedale_gpio_sd_regulator_dev,
};

static struct platform_device *regulator_devices_TI[] __initdata = {
	&vprog3_device,
	&anniedale_gpio_sd_regulator_dev,
};

static int __init regulator_init(void)
{
	int gpio = get_gpio_by_name("SDIO_PWR_EN");
	hw_rev revision;

	/* Always use GPIO number exported by IFWI */
	if (gpio > 0)
		anniedale_gpio_sd_regulator_data.gpio = gpio;
	else
		pr_warn("IFWI doesn't export SDIO_PWR_EN, using default: %d\n",
			anniedale_gpio_sd_regulator_data.gpio);

	/* Register the regulator only if SoC is Tangier or Anniedale */
	if (intel_mid_identify_cpu() == INTEL_MID_CPU_CHIP_TANGIER ||
		intel_mid_identify_cpu() == INTEL_MID_CPU_CHIP_ANNIEDALE) {
		revision = asustek_get_hw_rev();
		switch (revision) {
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
		case 6:
			printk(KERN_INFO "regulator_devices_NXP\n");
			platform_add_devices(regulator_devices_NXP,
				ARRAY_SIZE(regulator_devices_NXP));
			break;
		case 7:
			printk(KERN_INFO "regulator_devices_TI\n");
			platform_add_devices(regulator_devices_TI,
				ARRAY_SIZE(regulator_devices_TI));
			break;
		default:
			printk(KERN_INFO "Unknown HW_ID.\n");
		}
	}

	return 0;
}
device_initcall(regulator_init);
